package com.mycompany.award;

//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*
//*--------------------------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability, 
//* serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands and products 
//* in order to illustrate concepts as completely as possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by actual persons 
//* or business enterprises is entirely coincidental.
//*--------------------------------------------------------------------------------------

import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.payment.commands.*;

import java.util.*;
import java.math.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;

import com.ibm.commerce.order.objects.*;

import com.ibm.commerce.copyright.IBMCopyright;
/**
 * The <code>DoCancelPolicyCmd</code> implementation class for the PayWithAward policy. 
 */
public class DoCancelAwardCmdImpl extends BusinessPolicyCommandImpl implements DoCancelPolicyCmd 
{
	private final static String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;
	private final static String CLASSNAME = "com.mycompany.award.DoCancelAwardCmdImpl";

    private OrderAccessBean iabOrder                = null;
	/**
	 * DoCancelAwardCmdImpl default constructor.
	 */
	public DoCancelAwardCmdImpl() {
		super();
	}
	/**
	 *  Returns the OrderAccessBean representing the Order.  This is the OrderAccessBean passed in to the command.
	 */
	protected OrderAccessBean getOrder () 
	{
		return iabOrder;
	}
	/**
	 * Performs the main business logic of the command.
	 */
	public void performExecute() throws com.ibm.commerce.exception.ECException 
	{
		final String methodName = "performExecute";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		try {
			rollbackAwardPoint();
		} catch( javax.ejb.CreateException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_CREATE_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.ejb.FinderException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_FINDER_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.naming.NamingException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_NAMING_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( java.rmi.RemoteException ex ) {
			throw new ECSystemException(
					ECMessage._ERR_REMOTE_EXCEPTION, CLASSNAME, methodName, ex );
		}
	
		
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
	protected void rollbackAwardPoint() throws FinderException, CreateException, NamingException, RemoteException, ECApplicationException
	{
		final String methodName = "rollbackAwardPoint";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		OrderAccessBean abOrder = getOrder();

		OrderItemAccessBean[] orderItems = abOrder.getOrderItems();
		String tradingId = orderItems[0].getContractId();

		// Make sure all order items are purchased under the same contract (required by this example)
		for (int i=1; i < orderItems.length; i++ ) {
			if (!tradingId.equalsIgnoreCase(orderItems[i].getContractId()) ) {
				ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_ORDERITEMS_FROM_MULTIPLE_CONTRACTS", "award");
				throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
			}
		}

		// Look-up the Terms & Conditions object
		TradingAgreementAccessBean abTA = new TradingAgreementAccessBean();
		abTA.setInitKey_tradingId(tradingId);
		TermConditionAccessBean[] tcs = abTA.getTCsByTCSubType("AwardRedemptionTC", getUserId());
	
		if ( (tcs != null) && (tcs.length !=0) ) {
			
			// get the AwardRedemptionTC
			AwardRedemptionTCAccessBean abAwardRedemptionTC = new AwardRedemptionTCAccessBean();
			abAwardRedemptionTC.setInitKey_referenceNumber(tcs[0].getReferenceNumber());
	
			String cur = abAwardRedemptionTC.getCurrency();
			if ( cur.equalsIgnoreCase(abOrder.getCurrency()) ) {
				// OrderTotal = ProductTotal + ProductTax  + ShippingTotal + ShippingTax + Adjustment
				BigDecimal dOrderTotal = abOrder.getTotalProductPriceInEJBType();
				dOrderTotal = dOrderTotal.add( abOrder.getTotalTaxInEJBType() );
				dOrderTotal = dOrderTotal.add( abOrder.getTotalShippingChargeInEJBType() );
				dOrderTotal = dOrderTotal.add( abOrder.getTotalShippingTaxInEJBType() );
				dOrderTotal = dOrderTotal.add( abOrder.getTotalAdjustmentInEJBType() );		
	
				// Get the ratio of how many points are awarded per amount spent
				BigDecimal amount = abAwardRedemptionTC.getValueInEJBType();
				Integer points = abAwardRedemptionTC.getPointInEJBType();
				
				int pointsCharged = BigDecimal.valueOf(points.longValue(), 0).multiply(dOrderTotal).divide(amount, 0, BigDecimal.ROUND_CEILING).intValue();  

				// put the deducted award points back
				AwardAccessBean abAward = new AwardAccessBean();
				abAward.setInitKey_memberId(getUserId().toString());
				abAward.setInitKey_tradingId(tradingId);

				int currentPoint = abAward.getPointInEJBType().intValue();
				int newPoint = currentPoint + pointsCharged;

				abAward.setPoint(new Integer(newPoint));
				abAward.commitCopyHelper();
				
			} else {
				ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_CURRENCY_NOT_MATCH_WITH_AWARD_REDEMPTION_TC", "award");
				throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
			}
		} else {
			ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_NO_AWARD_REDEMPTION_TC", "award");
			throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
		}
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
	/**
	 * Optional error view name passed in by the Caller.
	 */
	public void setErrorViewName(String sEVN) {}
	/**
	 * The order to be processed.
	 */
	public void setOrder(OrderAccessBean abOrder) 
	{
		iabOrder = abOrder;
	}
}
