package com.mycompany.award;

//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*
//*--------------------------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability, 
//* serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands and products 
//* in order to illustrate concepts as completely as possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by actual persons 
//* or business enterprises is entirely coincidental.
//*--------------------------------------------------------------------------------------

import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import java.util.*;
import java.sql.*;
import java.math.*;

import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.payment.commands.*;
import com.ibm.commerce.payment.extern.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;

import com.ibm.commerce.copyright.IBMCopyright;

public class DoDepositWithAwardCollectionCmdImpl extends DoDepositCmdImpl
{
	private final static String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;
	private final static String CLASSNAME = "com.mycompany.award.DoDepositWithAwardCollectionCmdImpl";

	/**
	 * Calculate the award points
	 */
	protected void addAwardPoint() throws FinderException, CreateException, NamingException, RemoteException, ECApplicationException 
	{
		final String methodName = "addAwardPoint";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
		
		OrderAccessBean abOrder = getOrder();
		
		OrderItemAccessBean[] orderItems = abOrder.getOrderItems();
		String tradingId = orderItems[0].getContractId();

		// Make sure all order items are purchased under the same contract (required by this example)
		for (int i=1; i < orderItems.length; i++ ) {
			if (!tradingId.equalsIgnoreCase(orderItems[i].getContractId()) ) {
				ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_ORDERITEMS_FROM_MULTIPLE_CONTRACTS", "award");
				throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
			}
		}

		// Look-up the Terms & Conditions object
		TradingAgreementAccessBean abTA = new TradingAgreementAccessBean();
		abTA.setInitKey_tradingId(tradingId);
		TermConditionAccessBean[] tcs = abTA.getTCsByTCSubType("AwardCollectionTC");

		if ( (tcs != null) && (tcs.length != 0) ) {
			AwardCollectionTCAccessBean abAwardCollectionTC = new AwardCollectionTCAccessBean();
			abAwardCollectionTC.setInitKey_referenceNumber(tcs[0].getReferenceNumber());
			String cur = abAwardCollectionTC.getCurrency();
	
			// Calculate the award points
			BigDecimal amount = abAwardCollectionTC.getValueInEJBType();
			Integer points = abAwardCollectionTC.getPointInEJBType();
			int incrementalPoint = BigDecimal.valueOf(points.longValue(), 0).multiply(getDepositAmount()).divide(amount, 0, BigDecimal.ROUND_FLOOR).intValue();  
	
			// Try to add an entry to award table
			try {
				AwardAccessBean abAward = new AwardAccessBean(new Long(getOrder().getMemberId()), new Long(tradingId));
				abAward.setPoint(new Integer(incrementalPoint));
				abAward.commitCopyHelper();
			} catch (javax.ejb.CreateException ex) {
				// It already exists, so update
				AwardAccessBean abAward = new AwardAccessBean();
				abAward.setInitKey_memberId(getOrder().getMemberId());
				abAward.setInitKey_tradingId(tradingId);

				int currentPoint = abAward.getPointInEJBType().intValue();
				int newPoint = currentPoint + incrementalPoint;

				abAward.setPoint(new Integer(newPoint));
				abAward.commitCopyHelper();
			}
		}
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}

	/**
	 * Performs the main business logic of the command.
	 */
	public void performExecute() throws ECException 
	{
		final String methodName = "performExecute";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
	
		try {
			super.performExecute();
			addAwardPoint();	

		} catch( javax.ejb.CreateException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_CREATE_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.ejb.FinderException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_FINDER_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.naming.NamingException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_NAMING_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( java.rmi.RemoteException ex ) {
			throw new ECSystemException(
					ECMessage._ERR_REMOTE_EXCEPTION, CLASSNAME, methodName, ex );
		}
	
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}	
}