package com.mycompany.award;

//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*
//*--------------------------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability, 
//* serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands and products 
//* in order to illustrate concepts as completely as possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by actual persons 
//* or business enterprises is entirely coincidental.
//*--------------------------------------------------------------------------------------

import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import java.sql.*;
import java.math.*;
import java.util.*;
import java.util.Date;

import com.ibm.commerce.payment.commands.*;
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.order.commands.OrderProcessCmdImpl;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.payment.objects.*;	
import com.ibm.commerce.order.objimpl.*;

/**
 * This <code>DoPaymentAwardCmdImpl</code> Task Command Implementation is called by 
 * the <code>OrderProcessCmd</code> Controller Command to process payment 
 * for an order.
 * <p>
 * If the command is successful, the <code>OrderProcessCmd</code> will set the Order
 * status to the value specified by this command.
 * <p>
 * If unsuccessful, the command throws the ECApplicationException
 * specifying an Error Code and the Error View named by the constant
 * com.ibm.commerce.DoPaymentCmd.ERROR_TASK_NAME.
 * <p>
 * This implementation of the <code>DoPaymentCmd</code> Task Command does 
 * not have any returnData for the <code>OrderProcessCmd</code>.
 *
 */
public class DoPaymentAwardCmdImpl  extends BusinessPolicyCommandImpl implements DoPaymentPolicyCmd
{
	private final static String COPYRIGHT 				= IBMCopyright.SHORT_COPYRIGHT;
	private final static String CLASSNAME 				= "com.mycompany.award.DoPaymentAwardCmdImpl";

	private final static int    MAX_PAYMETHOD_LEN   	= 5;
		
	private Long            	itsOrderId				= null;
	private OrderAccessBean 	iabOrder                = null;
	private AccountAccessBean 	iabAccount            	= null;	
	private BigDecimal 			itsTotalAmount  		= null;
	private TypedProperty 		itsPaymentAttributes    = null;
	
	private Long            	inBuyerPOId             = null;
	private String          	isErrorViewName         = ERRTASK_NAME;

	private String          	istrOrderState          = null;
	
	/**
	 * Checks the validity of customer-supplied parameters.
	 */
	protected void checkParameters() throws ECException
	{
		final String methodName = "checkParameters";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		// Check for User Input Error and throws ECApplicationException....
		//
		TypedProperty paymentAttributes = getPaymentAttributes();
			
	
	
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
		public AccountAccessBean getAccount () {return iabAccount;}
		protected Long getAccountId ()
				throws javax.ejb.CreateException,javax.ejb.FinderException,
						javax.naming.NamingException,java.rmi.RemoteException {
			if (getAccount() == null) {
				return null;
			} else {
				return getAccount().getAccountIdInEJBType();
			}
		}
		protected Long getBuyerPOId () {
			return inBuyerPOId;
		}
	/**
	 * Returns the Content Type (or MIME type) of the return data, or <code>null</code> if
	 * the type is not known.
	 *
	 * @return a <code>String</code> containing the name of the MIME type of the return data, 
	 * or null if the type is not known.  Since this command implementation does not have
	 * return data, the method returns <code>null</code>.
	 */
	public String getContentType() 
	{
		return null;
	}
	protected String getErrorViewName () { return isErrorViewName; }
	/**
	 * This method will first check if the local variable is set or not.
	 * If not set it will attempt to get it by create a new AccessBean.
	 */
	protected OrderAccessBean getOrder () {
		return iabOrder;
	}
	/**
	 * Returns the single character String value that the called command
	 * wants the Calling command to use to set the Status of the order.
	 *
	 * If the returned String is empty or null, the Calling command will 
	 * decide the value to use for the Status
	 *
	 * @return java.lang.String
	 */
	public String getOrderState() 
	{
		return istrOrderState;
	}
	protected TypedProperty getPaymentAttributes()
	{
		return itsPaymentAttributes;
	}
	/**
	 * Returns the byte Array containing Return Data from the Payment System.
	 * 
	 * The Payment System used by this command does not use Return Data.  Therefore return null.
	 * 
	 * @return byte[]
	 */
	public byte[] getReturnData() 
	{
		return null;
	}
	protected BigDecimal getTotalAmount()
	{
		return itsTotalAmount;
	}
	/**
	 * This method is called by the Command Framework to check if all basic mandatory
	 * parameters have been set for this command.  This is done before the Framework calls
	 * the performExecute method of the command.
	 */
	public boolean isReadyToCallExecute() 
	{
		final String methodName = "isReadyToCallExecute";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		if ( getOrder() == null )
		{
			ECMessageLog.out( ECMessage._ERR_DIDNT_SET_ORDER, CLASSNAME, methodName );
			return false;
		}
		if ( getTotalAmount() == null )
		{
			ECMessageLog.out( ECMessage._ERR_DIDNT_SET_TOTAL_AMOUNT, CLASSNAME, methodName );
			return false;
		}
	
		if ( getPaymentAttributes()==null )
		{
			ECMessageLog.out( ECMessage._ERR_DIDNT_SET_PAYMENT_ATTRIB, CLASSNAME, methodName );
			return false;
		}
	
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
		return true;
	}
	
	
	
	/**
	 * Checks the available award points, substracts the points to do the payment, and updates the order status
	 */
	protected void payWithAwardPoint() throws FinderException, CreateException, NamingException, RemoteException, ECApplicationException
	{
		final String methodName = "payWithAwardPoint";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		OrderAccessBean abOrder = getOrder();

		OrderItemAccessBean[] orderItems = abOrder.getOrderItems();
		String tradingId = orderItems[0].getContractId();
		
		// Make sure all order items are purchased under the same contract (required by this example)
		for (int i=1; i < orderItems.length; i++ ) {
			if (!tradingId.equalsIgnoreCase(orderItems[i].getContractId()) ) {
				ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_ORDERITEMS_FROM_MULTIPLE_CONTRACTS", "award");
				throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
			}
		}
		
		// Look-up the Terms & Conditions object
		TradingAgreementAccessBean abTA = new TradingAgreementAccessBean();
		abTA.setInitKey_tradingId(tradingId);
		TermConditionAccessBean[] tcs = abTA.getTCsByTCSubType("AwardRedemptionTC", getUserId());
	
		if ( (tcs != null) && (tcs.length !=0) ) {
			
			// get order amount and currency
			String orderCur = abOrder.getCurrency();
	
			// get the AwardRedemptionTC
			AwardRedemptionTCAccessBean abAwardRedemptionTC = new AwardRedemptionTCAccessBean();
			abAwardRedemptionTC.setInitKey_referenceNumber(tcs[0].getReferenceNumber());
			
			String cur = abAwardRedemptionTC.getCurrency();
			
			if ( cur.equalsIgnoreCase(orderCur) ) {

				// Calculate how many points are needed to pay for the order
				BigDecimal amount = abAwardRedemptionTC.getValueInEJBType();
				Integer points = abAwardRedemptionTC.getPointInEJBType();
				int pointNeeded = BigDecimal.valueOf(points.longValue(), 0).multiply(itsTotalAmount).divide(amount, 0, BigDecimal.ROUND_CEILING).intValue();

				// Check how many points are available 
				AwardAccessBean abAward = new AwardAccessBean();
				abAward.setInitKey_memberId(getUserId().toString());
				abAward.setInitKey_tradingId(tradingId);
				int currentPoint = abAward.getPointInEJBType().intValue();
				
				if (currentPoint > pointNeeded ) {
					
					// Substract the points
					int newPoint = currentPoint - pointNeeded;
					abAward.setPoint(new Integer(newPoint));
					abAward.commitCopyHelper();
					ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName,
						"This purchase uses the TestPaymentMethod.  OrderState to be returned: " 
						+ OrderConstants.ORDER_COMPLETED );
			
					// Update order status to 'C' (completed)
					setOrderState( OrderConstants.ORDER_COMPLETED );					

				} else {					
					ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_AWARD_POINTS_NOT_ENOUGH", "award");
					throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
				}
			} else {				
				ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_CURRENCY_NOT_MATCH_WITH_AWARD_REDEMPTION_TC", "award");
				throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
			}
		} else {
			ECMessage ecm = new ECMessage(ECMessageSeverity.ERROR, ECMessageType.USER, "_ERR_NO_AWARD_REDEMPTION_TC", "award");
			throw new ECApplicationException(ecm, this.getClass().getName(), methodName);	
		}
		
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
	/**
	 * Performs the main business logic of the command.
	 */
	public void performExecute() throws ECException {
		final String methodName = "performExecute";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );

		try {

			// Create new OrdPayMthd object
			String strPayMethod = "AWARD";
			OrderPaymentMethodInputData bOrderPaymentMethodInputData = new OrderPaymentMethodInputData();			
			bOrderPaymentMethodInputData.setOrderId(getOrder().getOrderIdInEJBType());
			bOrderPaymentMethodInputData.setPaymentMethodId(strPayMethod);
			bOrderPaymentMethodInputData.setPaymentDevice("0");
			bOrderPaymentMethodInputData.setMaximumAuthorizationAmount(getTotalAmount());
			bOrderPaymentMethodInputData.setAccountId(getAccountId());
			bOrderPaymentMethodInputData.setBuyerPurchaseOrderId(getBuyerPOId());
			bOrderPaymentMethodInputData.setPolicyId(getPolicyId());

			OrderPaymentMethodAccessBean abOrdPayMthd = new OrderPaymentMethodAccessBean(bOrderPaymentMethodInputData, false);
			
			ECTrace.trace( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName, 
				"Returned from OrdPayMthdAccessBeam.commitCopyHelper() ...." );

			// Update AWARD table
			payWithAwardPoint();
			
		} catch( javax.ejb.CreateException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_CREATE_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.ejb.FinderException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_FINDER_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( javax.naming.NamingException ex ) {
			throw new ECSystemException( 
					ECMessage._ERR_NAMING_EXCEPTION, CLASSNAME, methodName, ex );
		} catch( java.rmi.RemoteException ex ) {
			throw new ECSystemException(
					ECMessage._ERR_REMOTE_EXCEPTION, CLASSNAME, methodName, ex );
		}
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}
	
	/**
	 * Resets the instance variables of the command.
	 * The reset() method will reset all instance variables.
	 * If the command includes any output values that are accessible via 
	 * getters, then make sure those values are no longer needed before calling
	 * this method to reset the instance variables of the command.  
	 * <p>
	 * This is useful if the command instance is to be called multiple times with
	 * different command parameters.
	 * <p>
	 */
	public void reset() 
	{
		itsOrderId		= null;
		itsTotalAmount  	= null;
		itsPaymentAttributes    = null;
		
	}
	public void setAccount (AccountAccessBean abAccount) {iabAccount=abAccount;}
	/**
	 * Set Optional buyerPOId.
	 */
	public void setBuyerPOId(Long n) {
		inBuyerPOId = n;
	}
	/**
	 * Optional error view name passed in by the Caller.
	 */
	public void setErrorViewName (String sEVN) { isErrorViewName = sEVN;}
	/**
	 * The order to be ReApproved.
	 */
	public void setOrder (OrderAccessBean ab) {
		iabOrder = ab;
	}
	protected void setOrderState(String orderState) 
	{ 
		istrOrderState = orderState; 
	}
	/**
	 * Sets the paymentAttributes property of the command.
	 * 
	 * @param requestProperties the requestProperties object that contains all
	 * the command parameters are to be passed to this command.
	 *
	 * <P>
	 * This command requires the following Payment Attributes:
	 * <PRE>
	 *       Keyword                         Value
	 *     ===========        ====================================
	 *     cardBrand          The credit card brand.  Represents the credit card type used.
	 *     cardNumber         The credit card number              
	 *     cardExpiryMonth    The credit card exipration month  
	 *     cardExpiryYear     The credit card exipration year   
	 * </PRE>
	 * Note that this command stores the credit card brand in the PAYMETHOD column of 
	 * the ORDPAYMTHD table and the size of the column is limited to five characters long.
	 * 
	 */
	public void setPaymentAttributes(TypedProperty paymentAttributes)
	{
		itsPaymentAttributes = paymentAttributes;
	}
	/** 
	 * Sets the totalAmount property of the command.
	 * 
	 * @param totalAmount the total price of the Order for which payment
	 * is to be processed.
	 */
	public void setTotalAmount(BigDecimal totalAmount)
	{
		itsTotalAmount=totalAmount;
	}
	/**
	 * Checks the validity of customer-supplied parameters.
	 */
	public void validateParameters() throws ECException
	{
		final String methodName = "validateParameters";
		ECTrace.entry( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	
		// Check for User Input Error and throws ECApplicationException....
	
		ECTrace.exit( ECTraceIdentifiers.COMPONENT_ORDER, CLASSNAME, methodName );
	}

}
